(**************************************************************************)
(*  This file is part of the TAI abstract interpreter.                    *)
(*                                                                        *)
(*  Copyright (C) 2024                                                    *)
(*    CEA (Commissariat à l'énergie atomique et aux énergies              *)
(*         alternatives)                                                  *)
(*                                                                        *)
(*  You can redistribute it and/or modify it under the terms of the GNU   *)
(*  Lesser General Public License as published by the Free Software       *)
(*  Foundation, version 2.1.                                              *)
(*                                                                        *)
(*  It is distributed in the hope that it will be useful,                 *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *)
(*  GNU Lesser General Public License for more details.                   *)
(*                                                                        *)
(*  See the GNU Lesser General Public License version 2.1                 *)
(*  for more details (enclosed in the file LICENSE).                      *)
(**************************************************************************)

(** Declares options for the command line interface *)

type display =
  | InputCFG
  | InputGraph
  | WTO
  | WideningEdges
  | ConvergencePoint
  | OutCFG
  | OutGraph
  | Summary
  | CSV

val out_graph_file : string
val in_graph_file : string

type arguments = {
  enabled : bool;
  iterative_strategy : bool;
      (** Wether or not to use an iterative strategy in wto.ml (stabilize outer loops first) *)
  program : Ast.Program.graph;
      (** The program to analysed, parsed by Frama-C and tranlated to our AST in ast/program.ml *)
  numeric : (module Single_value_abstraction.Sig.SingleValueAbstraction);
      (** The numerical domain to use for the analysis*)
  domain : Domain_builder.classical_domain;  (** The main analysis domain *)
  nb_backwards_iterations : int;
      (** The number of backward iterations performed when back-propagating expressions in
          domains/numeric.ml *)
  display : display -> bool;  (** Should the relevant display be printed *)
}

val get_args : unit -> arguments
