(**************************************************************************)
(*  This file is part of the TAI abstract interpreter.                    *)
(*                                                                        *)
(*  Copyright (C) 2024                                                    *)
(*    CEA (Commissariat à l'énergie atomique et aux énergies              *)
(*         alternatives)                                                  *)
(*                                                                        *)
(*  You can redistribute it and/or modify it under the terms of the GNU   *)
(*  Lesser General Public License as published by the Free Software       *)
(*  Foundation, version 2.1.                                              *)
(*                                                                        *)
(*  It is distributed in the hope that it will be useful,                 *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *)
(*  GNU Lesser General Public License for more details.                   *)
(*                                                                        *)
(*  See the GNU Lesser General Public License version 2.1                 *)
(*  for more details (enclosed in the file LICENSE).                      *)
(**************************************************************************)

(** Simple lexer for the -tai-domain command line argument *)

{
(** Simple lexer for the [-tai-domain] command line argument *)

  open Cli_parser

  let lexing_error message =
    Std_extra.Utils.printf "@{<bold>@{<red>LEXING ERROR@}: %s@}@." message;
    exit 8
}

let ident = ['A'-'Z']+
let space = ' ' | '\t'
let prod = '*' | 'x'

rule tokens = parse
  | '\n'            { Lexing.new_line lexbuf; tokens lexbuf }
  | space+          { tokens lexbuf }
  | ident as id     { Ident id }
  | "G" | "LiftGVN" { Lift true }
  | "L" | "Lift"    { Lift false }
  | "("             { LeftParen }
  | ")"             { RightParen }
  | "["             { LeftBracket }
  | "]"             { RightBracket }
  (* | prod "b" | '$'  { Product Both }
  | prod "l" | '%'  { Product Left } *)
  | prod            { Product (*Basic*) }
  | eof             { EOF }
  | _ as c          { lexing_error ("illegal character: '" ^ String.make 1 c ^
  "' in -tai-domain\n  Expected an uppercase identifier, '(', ')', '*', 'Lift' or 'LiftGVN'.") }
