/**************************************************************************/
/*  This file is part of the TAI abstract interpreter.                    */
/*                                                                        */
/*  Copyright (C) 2024                                                    */
/*    CEA (Commissariat à l'énergie atomique et aux énergies              */
/*         alternatives)                                                  */
/*                                                                        */
/*  You can redistribute it and/or modify it under the terms of the GNU   */
/*  Lesser General Public License as published by the Free Software       */
/*  Foundation, version 2.1.                                              */
/*                                                                        */
/*  It is distributed in the hope that it will be useful,                 */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of        */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         */
/*  GNU Lesser General Public License for more details.                   */
/*                                                                        */
/*  See the GNU Lesser General Public License version 2.1                 */
/*  for more details (enclosed in the file LICENSE).                      */
/**************************************************************************/

/* Simple parser for the -tai-domain command line argument  */

%{
  (** Simple parser for the -tai-domain command line argument *)

  open Domain_builder
%}

%token LeftParen RightParen LeftBracket RightBracket
%token <string> Ident
%token <bool> Lift
%token Product

%left Product
%token EOF

/* Point d'entrée de la grammaire */
%start full_expr

/* Type des valeurs renvoyées par l'analyseur syntaxique */
%type <Domain_builder.classical_domain> full_expr

%%

full_expr:
| domain = classical_domain EOF { domain };

classical_domain:
| LeftParen domain = classical_domain RightParen                 { domain }
| id = Ident                                                     { Classical_Domain id }
| fn = Ident LeftParen domain = classical_domain RightParen      { Classical_Functor(fn, domain) }
| dom = Ident LeftBracket n = Ident RightBracket                 { Classical_Numeric(dom, n) }
| left = classical_domain Product right = classical_domain  { Classical_Product(left,right) }
| gvn = Lift LeftParen domain = ssa_domain RightParen            { Lift(gvn,domain) }
;

ssa_domain:
| LeftParen domain = ssa_domain RightParen             { domain }
| id = Ident                                           { SSA_Domain id }
| dom = Ident LeftBracket n = Ident RightBracket       { SSA_Numeric(dom, n) }
| fn = Ident LeftParen domain = ssa_domain RightParen  { SSA_Functor(fn, domain) }
| left = ssa_domain Product right = ssa_domain    { SSA_Product(left,right) }
;
