(**************************************************************************)
(*  This file is part of the TAI abstract interpreter.                    *)
(*                                                                        *)
(*  Copyright (C) 2024                                                    *)
(*    CEA (Commissariat à l'énergie atomique et aux énergies              *)
(*         alternatives)                                                  *)
(*                                                                        *)
(*  You can redistribute it and/or modify it under the terms of the GNU   *)
(*  Lesser General Public License as published by the Free Software       *)
(*  Foundation, version 2.1.                                              *)
(*                                                                        *)
(*  It is distributed in the hope that it will be useful,                 *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *)
(*  GNU Lesser General Public License for more details.                   *)
(*                                                                        *)
(*  See the GNU Lesser General Public License version 2.1                 *)
(*  for more details (enclosed in the file LICENSE).                      *)
(**************************************************************************)

(** Simple transformation functors *)

open Sig

module Ternary : Simple_Transformation
(** Since Frama-c's parser already removes ternary expressions, this isn't used.
    Removes [Ast.Expr.E_IfThenElse] constructs by making the control flow jumps
    explicit.

    Its command line id is [T]. *)

module DivGuard : Simple_Transformation
(** Add a guard [divisor <> 0] before every division.

    Its command line id is [D].*)

module ShiftGuard : Simple_Transformation
(** Add a guard [shift_ammount >= 0] before every shift.

    Its command line id is [S]. *)

module Query : Simple_Transformation
(** Simplification via simple queries. This functor is often inlined for
    efficiency, as in {!Product.ClassicalQueryLeft} or {!Numeric.ClassicalQuery}.

    Its command line id is [Q]. *)

module SimplifyExpression : Simple_Transformation
(** Simple expression rewrites using queries (x + 0 -> x and others).
    This domain isn't complete: it will rewrite things like x = x -> 1
    which can cause problems if x contains a division by 0.

    Its command line id is [E]. *)

module SSA_SimplifyExpression : SSA_Transformation
(** SSA variant of {!SimplifyExpression}.

    Its command line id is [E] (under [Lift] or [LiftGVN]) *)

module SSA_Query : SSA_Transformation
(** Simplification via simple queries. This functor is often inlined for
    efficiency, as in {!Product.SSAQueryLeft} or {!Numeric.SSAQuery}.

    Its command line id is [Q] (under [Lift] or [LiftGVN]) *)
