(**************************************************************************)
(*  This file is part of the TAI abstract interpreter.                    *)
(*                                                                        *)
(*  Copyright (C) 2024                                                    *)
(*    CEA (Commissariat à l'énergie atomique et aux énergies              *)
(*         alternatives)                                                  *)
(*                                                                        *)
(*  You can redistribute it and/or modify it under the terms of the GNU   *)
(*  Lesser General Public License as published by the Free Software       *)
(*  Foundation, version 2.1.                                              *)
(*                                                                        *)
(*  It is distributed in the hope that it will be useful,                 *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *)
(*  GNU Lesser General Public License for more details.                   *)
(*                                                                        *)
(*  See the GNU Lesser General Public License version 2.1                 *)
(*  for more details (enclosed in the file LICENSE).                      *)
(**************************************************************************)

(** Single value abstraction using intervals *)

type t = private {
  low : Z.t option; (* Use NONE for empty *)
  high : Z.t option;
}
(** A representation of intervals,
    [None] bound means plus/minus infinity.
    Invariant: [low <= high] (if neither are [None]) *)

val interval : Z.t option -> Z.t option -> t option
(** [interval low high] is a contructors for type [t], ensures the invariant is respected *)

val remove_const : t -> Z.t -> t option
(** [remove_const interval value] returns the best abtraction for interval \ value
    (where \ is setminus). It returns [None] if that abstraction is bottom. *)

include Sig.SingleValueAbstraction with type t := t
