(**************************************************************************)
(*  This file is part of the TAI abstract interpreter.                    *)
(*                                                                        *)
(*  Copyright (C) 2024                                                    *)
(*    CEA (Commissariat à l'énergie atomique et aux énergies              *)
(*         alternatives)                                                  *)
(*                                                                        *)
(*  You can redistribute it and/or modify it under the terms of the GNU   *)
(*  Lesser General Public License as published by the Free Software       *)
(*  Foundation, version 2.1.                                              *)
(*                                                                        *)
(*  It is distributed in the hope that it will be useful,                 *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *)
(*  GNU Lesser General Public License for more details.                   *)
(*                                                                        *)
(*  See the GNU Lesser General Public License version 2.1                 *)
(*  for more details (enclosed in the file LICENSE).                      *)
(**************************************************************************)

(** Combine single value abstraction using (reduced) products*)

open Sig

(** Create a product abstraction, using the given reduction function *)
module Make
    (L : SingleValueAbstraction)
    (R : SingleValueAbstraction)
    (_ : sig
      val reduce : L.t * R.t -> L.t * R.t
    end) : SingleValueAbstraction with type t = L.t * R.t

module Interval_Sentinel : SingleValueAbstraction with type t = Interval.t * Sentinel.t
(** Product of {!Interval} and {!Sentinel} abstractions *)

module KnownBits_Sentinel : SingleValueAbstraction with type t = Known_bits.t * Sentinel.t
(** Product of {!Known_bits} and {!Sentinel} abstractions *)

module Interval_KnownBits_Sentinel :
  SingleValueAbstraction with type t = Interval.t * (Known_bits.t * Sentinel.t)
(** Product of {!Interval}, {!Known_bits} and {!Sentinel} abstractions *)
