(**************************************************************************)
(*  This file is part of the TAI abstract interpreter.                    *)
(*                                                                        *)
(*  Copyright (C) 2024                                                    *)
(*    CEA (Commissariat à l'énergie atomique et aux énergies              *)
(*         alternatives)                                                  *)
(*                                                                        *)
(*  You can redistribute it and/or modify it under the terms of the GNU   *)
(*  Lesser General Public License as published by the Free Software       *)
(*  Foundation, version 2.1.                                              *)
(*                                                                        *)
(*  It is distributed in the hope that it will be useful,                 *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *)
(*  GNU Lesser General Public License for more details.                   *)
(*                                                                        *)
(*  See the GNU Lesser General Public License version 2.1                 *)
(*  for more details (enclosed in the file LICENSE).                      *)
(**************************************************************************)

type t =
  | False
  | True
  | TrueFalse
[@@deriving eq, ord]

let order a b = Std_extra.Utils.order_of_compare (compare a b)

let pp fmt x =
  Format.fprintf fmt "%s"
    (match x with
    | False -> "False"
    | True -> "True"
    | TrueFalse -> "TrueFalse")

let hash = function
  | False -> 1
  | True -> 2
  | TrueFalse -> 3

let of_bool b = if b then True else False
let of_int i = if i = 0 then False else True
let of_z z = if Z.equal z Z.zero then False else True

let not = function
  | False -> True
  | True -> False
  | TrueFalse -> TrueFalse

let subseteq a b = b = TrueFalse || a = b

let ( || ) a b =
  match (a, b) with
  | True, _ | _, True -> True
  | False, False -> False
  | _, _ -> TrueFalse

let ( && ) a b =
  match (a, b) with
  | False, _ | _, False -> False
  | True, True -> True
  | _, _ -> TrueFalse

let join a b =
  match (a, b) with
  | True, True -> True
  | False, False -> False
  | _ -> TrueFalse

let meet a b =
  match (a, b) with
  | True, False -> None
  | True, _ -> Some True
  | False, True -> None
  | False, _ -> Some False
  | TrueFalse, b -> Some b
