(**************************************************************************)
(*  This file is part of the TAI abstract interpreter.                    *)
(*                                                                        *)
(*  Copyright (C) 2024                                                    *)
(*    CEA (Commissariat à l'énergie atomique et aux énergies              *)
(*         alternatives)                                                  *)
(*                                                                        *)
(*  You can redistribute it and/or modify it under the terms of the GNU   *)
(*  Lesser General Public License as published by the Free Software       *)
(*  Foundation, version 2.1.                                              *)
(*                                                                        *)
(*  It is distributed in the hope that it will be useful,                 *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *)
(*  GNU Lesser General Public License for more details.                   *)
(*                                                                        *)
(*  See the GNU Lesser General Public License version 2.1                 *)
(*  for more details (enclosed in the file LICENSE).                      *)
(**************************************************************************)

(** Simple abstraction of boolean values, including transfer function for
    the usual boolean operations. *)

open Std_extra.Types

type t =
  | False
  | True
  | TrueFalse

include TYPE with type t := t

val of_bool : bool -> t
val of_int : int -> t
val of_z : Z.t -> t

val not : t -> t
val ( || ) : t -> t -> t
val ( && ) : t -> t -> t

val join : t -> t -> t
val meet : t -> t -> t option
val subseteq : t -> t -> bool
