module Make (S : sig
  type t
  val pp : t Types.pp
  val compare : t -> t -> int
end) : sig
  include Types.TYPE_TO_INT

  val get_or_create : S.t -> t
end = struct
  type t = int * S.t

  let compare (a, _) (b, _) = Int.compare a b
  let equal (a, _) (b, _) = Int.equal a b
  let hash = fst
  let to_int = fst
  let order (a, _) (b, _) = if a < b then Types.Lt else if a = b then Types.Eq else Types.Gt
  let pp fmt (_, s) = S.pp fmt s

  module SMap = Map.Make (S)
  let count = ref 0
  let known = ref SMap.empty

  let get_or_create x =
    match SMap.find x !known with
    | elt -> elt
    | exception Not_found ->
        let elt = (!count, x) in
        known := SMap.add x elt !known;
        count := !count + 1;
        elt
end
